package com.redstrate.navigator

import NominatimApi
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.lifecycle.lifecycleScope
import com.redstrate.navigator.ui.theme.NavigatorTheme
import kotlinx.coroutines.launch
import okhttp3.Interceptor
import okhttp3.OkHttpClient
import okhttp3.Response
import retrofit2.Retrofit
import retrofit2.await
import retrofit2.converter.gson.GsonConverterFactory
import java.net.URLDecoder


class UserAgentInterceptor : Interceptor {
    override fun intercept(chain: Interceptor.Chain): Response {
        val originRequest = chain.request()
        val requestWithUserAgent = originRequest.newBuilder()
            .header("User-Agent", "redstrate's navigator")
            .build()
        return chain.proceed(requestWithUserAgent)
    }
}

class MainActivity : ComponentActivity() {
    private fun openMap(route: Boolean, latitude: String, longitude: String) {
        if (route) {
            // Unfortunately this is the best Organic Maps can do right now. You can't set it to "route" to a specific location, you HAVE to give it an explicit start and end.
            // In order to make this work, you need to tap the bottom footer to select "Current Location" (or any starting position) and then it all works out.
            val mapIntent: Intent = Uri.parse(
                "om://route?sll=${latitude},${longitude}&saddr=Start&dll=${latitude},${longitude}&daddr=End&type=transit"
            ).let { location ->
                Intent(Intent.ACTION_VIEW, location)
            }
            startActivity(mapIntent)
        } else {
            val mapIntent: Intent = Uri.parse(
                "om://map?v=1&ll=${latitude},${longitude}"
            ).let { location ->
                Intent(Intent.ACTION_VIEW, location)
            }
            startActivity(mapIntent)
        }
        finish()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        val data: Uri? = intent?.data
        if (data != null) {
            var query = data.toString()

            val route = query.contains("google.navigation")

            query = query.removePrefix("google.navigation:q=")
            query = query.removePrefix("geo:0,0?q=")
            query = query.removePrefix("geo:")

            if (query.isNotEmpty()) {
                val okHttp = OkHttpClient()
                okHttp.interceptors().add(UserAgentInterceptor())

                val retrofit = Retrofit.Builder()
                    .addConverterFactory(GsonConverterFactory.create())
                    .baseUrl("https://nominatim.openstreetmap.org")
                    .client(okHttp)
                    .build()

                val service: NominatimApi = retrofit.create(NominatimApi::class.java)

                val arguments = query.split(",")
                // A terrible way to check if this is lon/lat coordinates
                if (arguments.size == 2) {
                    // Clean stuff like -81.6848?z=12
                    val regex = Regex("([0-9-.]*)")

                    val latitude = regex.matchAt(arguments[0], 0)?.groups?.get(1)?.value
                    val longitude = regex.matchAt(arguments[1], 0)?.groups?.get(1)?.value

                    // This may not make sense, but we want to match it to an OSM id if available.
                    // Otherwise it will put a pin on the map without no clear indicator what it is we're looking at.
                    lifecycleScope.launch {
                        service.reverse(latitude!!, longitude!!).await()
                            .let { place: NominatimPlace ->
                                openMap(route, place.lat, place.lon)
                            }
                    }
                } else {
                    lifecycleScope.launch {
                        service.search(URLDecoder.decode(query)).await()
                            .let { places: List<NominatimPlace> ->
                                openMap(route, places.first().lat, places.first().lon)
                            }
                    }
                }

                setContent {
                    NavigatorTheme {
                        Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                            Greeting(
                                name = query,
                                modifier = Modifier.padding(innerPadding)
                            )
                        }
                    }
                }
                return
            }
        }
        setContent {
            NavigatorTheme {
                Scaffold(modifier = Modifier.fillMaxSize()) { innerPadding ->
                    Greeting(
                        name = "Navigator does nothing on it's own!",
                        modifier = Modifier.padding(innerPadding)
                    )
                }
            }
        }
    }
}

@Composable
fun Greeting(name: String, modifier: Modifier = Modifier) {
    Text(
        text = "Hello $name!",
        modifier = modifier
    )
}

@Preview(showBackground = true)
@Composable
fun GreetingPreview() {
    NavigatorTheme {
        Greeting("Android")
    }
}